#!/usr/bin/env python
#
# xkmap_widget.py   --  Simple GUI widget for xorg keymap configuring

# (c) Copyright 2009-2010 Michael Towers (larch42 at googlemail dot com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------
# 2010.08.14

class XkmapWidget:
    def __init__(self):
        self.widgetlist(self.fss('fetch_layout', 'xkmap-widget.uim'))


    def winit(self):
        self.models = self.fss('models')
        self.layouts = self.fss('layouts')
        startdata = self.fss('startdata')
        im = self.seekindex(self.models, startdata[0])
        il = self.seekindex(self.layouts, startdata[1])
        self.v0 = startdata[2]

        # Populate choice widgets
        self.command('xkmap:model.set',
                ['%-16s%s' % (m, c[:30]) for m, c in self.models], im)
        self.command('xkmap:layout.set',
                ['%-16s%s' % tuple(i) for i in self.layouts], il)
        self.newModel(im, False)
        self.newLayout(il)

        self.connectlist(
                ('xkmap:model*changed', self.newModel),
                ('xkmap:layout*changed', self.newLayout),
                ('xkmap:variant*changed', self.newVariant),
            )

    def newModel(self, index, show=True):
        self.model, tt = self.models[index]
        self.command('xkmap:model.tt', tt)
        if show:
            self.showCommand()


    def newLayout(self, index):
        self.layout = self.layouts[index][0]
        self.variants = self.fss('variantlist', self.layout)
        vi, i = 0, 0
        items = []
        for item0, item1 in self.variants:
            if len(item1) > 30:
                item1 = item1[:30]
            items.append("%-16s%s" % (item0, item1[:30]))
            if item0 == self.v0:
                vi = i
                tt = item1
            i += 1
        self.command('xkmap:variant.set', items, vi)
        self.newVariant(vi)


    def newVariant(self, index):
        self.variant, tt = self.variants[index]
        self.command('xkmap:variant.tt', tt)
        self.showCommand()


    def showCommand(self):
        self.command('xkmap:command.text', self.fss('command',
                self.model, self.layout, self.variant))


    def seekindex(self, tulist, element):
        """Return the index of the tuple in tulist whose first item
        is element.
        """
        i = 0
        for tu in tulist:
            if tu[0] == element:
                return i
            i += 1
        assert False, "Element not in list"

